@@ -2,7 +2,7 @@
 
 # ******************************************************************************
 #
-# Copyright (C) 2009-2010 Olivier Tilloy <olivier@tilloy.net>
+# Copyright (C) 2009-2011 Olivier Tilloy <olivier@tilloy.net>
 #
 # This file is part of the pyexiv2 distribution.
 #
@@ -68,11 +68,11 @@ class TestImageMetadata(unittest.TestCase):
     def test_not_read_raises(self):
         # http://bugs.launchpad.net/pyexiv2/+bug/687373
         self.assertRaises(IOError, self.metadata.write)
-        self.assertRaises(IOError, self.metadata.__getattribute__, 'dimensions')
-        self.assertRaises(IOError, self.metadata.__getattribute__, 'mime_type')
-        self.assertRaises(IOError, self.metadata.__getattribute__, 'exif_keys')
-        self.assertRaises(IOError, self.metadata.__getattribute__, 'iptc_keys')
-        self.assertRaises(IOError, self.metadata.__getattribute__, 'xmp_keys')
+        self.assertRaises(IOError, getattr, self.metadata, 'dimensions')
+        self.assertRaises(IOError, getattr, self.metadata, 'mime_type')
+        self.assertRaises(IOError, getattr, self.metadata, 'exif_keys')
+        self.assertRaises(IOError, getattr, self.metadata, 'iptc_keys')
+        self.assertRaises(IOError, getattr, self.metadata, 'xmp_keys')
         self.assertRaises(IOError, self.metadata._get_exif_tag, 'Exif.Image.Make')
         self.assertRaises(IOError, self.metadata._get_iptc_tag, 'Iptc.Application2.Caption')
         self.assertRaises(IOError, self.metadata._get_xmp_tag, 'Xmp.dc.format')
@@ -82,25 +82,25 @@ class TestImageMetadata(unittest.TestCase):
         self.assertRaises(IOError, self.metadata._delete_exif_tag, 'Exif.Image.Make')
         self.assertRaises(IOError, self.metadata._delete_iptc_tag, 'Iptc.Application2.Caption')
         self.assertRaises(IOError, self.metadata._delete_xmp_tag, 'Xmp.dc.format')
-        self.assertRaises(IOError, self.metadata._get_comment)
-        self.assertRaises(IOError, self.metadata._set_comment, 'foobar')
-        self.assertRaises(IOError, self.metadata._del_comment)
-        self.assertRaises(IOError, self.metadata.__getattribute__, 'previews')
+        self.assertRaises(IOError, getattr, self.metadata, 'comment')
+        self.assertRaises(IOError, setattr, self.metadata, 'comment', 'foobar')
+        self.assertRaises(IOError, delattr, self.metadata, 'comment')
+        self.assertRaises(IOError, getattr, self.metadata, 'previews')
         other = ImageMetadata(self.pathname)
         self.assertRaises(IOError, self.metadata.copy, other)
-        self.assertRaises(IOError, self.metadata.__getattribute__, 'buffer')
+        self.assertRaises(IOError, getattr, self.metadata, 'buffer')
         thumb = self.metadata.exif_thumbnail
-        self.assertRaises(IOError, thumb.__getattribute__, 'mime_type')
-        self.assertRaises(IOError, thumb.__getattribute__, 'extension')
+        self.assertRaises(IOError, getattr, thumb, 'mime_type')
+        self.assertRaises(IOError, getattr, thumb, 'extension')
         self.assertRaises(IOError, thumb.write_to_file, '/tmp/foobar.jpg')
         self.assertRaises(IOError, thumb.erase)
         self.assertRaises(IOError, thumb.set_from_file, '/tmp/foobar.jpg')
-        self.assertRaises(IOError, thumb.__getattribute__, 'data')
-        self.assertRaises(IOError, thumb.__setattr__, 'data', EMPTY_JPG_DATA)
-        self.assertRaises(IOError, self.metadata.__getattribute__, 'iptc_charset')
+        self.assertRaises(IOError, getattr, thumb, 'data')
+        self.assertRaises(IOError, setattr, thumb, 'data', EMPTY_JPG_DATA)
+        self.assertRaises(IOError, getattr, self.metadata, 'iptc_charset')
 
     def test_read(self):
-        self.assertRaises(IOError, self.metadata.__getattribute__, '_image')
+        self.assertRaises(IOError, getattr, self.metadata, '_image')
         self.metadata.read()
         self.failIfEqual(self.metadata._image, None)
 
@@ -564,6 +564,13 @@ class TestImageMetadata(unittest.TestCase):
         for key in keys:
             self.metadata[key] = self.metadata[key]        
 
+    def test_nonexistent_tag_family(self):
+        self.metadata.read()
+        key = 'Bleh.Image.DateTime'
+        self.failUnlessRaises(KeyError, self.metadata.__getitem__, key)
+        self.failUnlessRaises(KeyError, self.metadata.__setitem__, key, datetime.date.today())
+        self.failUnlessRaises(KeyError, self.metadata.__delitem__, key)
+
     ##########################
     # Test the image comment #
     ##########################
