@@ -2,7 +2,7 @@
 
 # ******************************************************************************
 #
-# Copyright (C) 2009-2010 Olivier Tilloy <olivier@tilloy.net>
+# Copyright (C) 2009-2011 Olivier Tilloy <olivier@tilloy.net>
 #
 # This file is part of the pyexiv2 distribution.
 #
@@ -34,6 +34,12 @@ from pyexiv2.metadata import ImageMetadata
 import datetime
 from testutils import EMPTY_JPG_DATA
 
+# Optional dependency on python-tz, more tests can be run if it is installed
+try:
+    import pytz
+except ImportError:
+    pytz = None
+
 
 class TestXmpTag(unittest.TestCase):
 
@@ -128,30 +134,70 @@ class TestXmpTag(unittest.TestCase):
         self.assertEqual(tag.type, 'Date')
         self.assertEqual(tag._convert_to_string(datetime.date(2009, 2, 4), 'Date'),
                          '2009-02-04')
+        self.assertEqual(tag._convert_to_string(datetime.date(1899, 12, 31), 'Date'),
+                         '1899-12-31')
         self.assertEqual(tag._convert_to_string(datetime.datetime(1999, 10, 13), 'Date'),
                          '1999-10-13')
+        self.assertEqual(tag._convert_to_string(datetime.datetime(1999, 10, 13, 5, 3), 'Date'),
+                         '1999-10-13T05:03Z')
+        self.assertEqual(tag._convert_to_string(datetime.datetime(1899, 12, 31, 23, 59), 'Date'),
+                         '1899-12-31T23:59Z')
         self.assertEqual(tag._convert_to_string(datetime.datetime(1999, 10, 13, 5, 3, tzinfo=FixedOffset()), 'Date'),
                          '1999-10-13T05:03Z')
+        self.assertEqual(tag._convert_to_string(datetime.datetime(1899, 12, 31, 23, 59, tzinfo=FixedOffset()), 'Date'),
+                         '1899-12-31T23:59Z')
         self.assertEqual(tag._convert_to_string(datetime.datetime(1999, 10, 13, 5, 3, tzinfo=FixedOffset('+', 5, 30)), 'Date'),
                          '1999-10-13T05:03+05:30')
         self.assertEqual(tag._convert_to_string(datetime.datetime(1999, 10, 13, 5, 3, tzinfo=FixedOffset('-', 11, 30)), 'Date'),
                          '1999-10-13T05:03-11:30')
+        self.assertEqual(tag._convert_to_string(datetime.datetime(1899, 12, 31, 23, 59, tzinfo=FixedOffset('+', 5, 30)), 'Date'),
+                         '1899-12-31T23:59+05:30')
+        self.assertEqual(tag._convert_to_string(datetime.datetime(1999, 10, 13, 5, 3, 27), 'Date'),
+                         '1999-10-13T05:03:27Z')
+        self.assertEqual(tag._convert_to_string(datetime.datetime(1899, 12, 31, 23, 59, 59), 'Date'),
+                         '1899-12-31T23:59:59Z')
         self.assertEqual(tag._convert_to_string(datetime.datetime(1999, 10, 13, 5, 3, 27, tzinfo=FixedOffset()), 'Date'),
                          '1999-10-13T05:03:27Z')
+        self.assertEqual(tag._convert_to_string(datetime.datetime(1899, 12, 31, 23, 59, 59, tzinfo=FixedOffset()), 'Date'),
+                         '1899-12-31T23:59:59Z')
         self.assertEqual(tag._convert_to_string(datetime.datetime(1999, 10, 13, 5, 3, 27, tzinfo=FixedOffset('+', 5, 30)), 'Date'),
                          '1999-10-13T05:03:27+05:30')
         self.assertEqual(tag._convert_to_string(datetime.datetime(1999, 10, 13, 5, 3, 27, tzinfo=FixedOffset('-', 11, 30)), 'Date'),
                          '1999-10-13T05:03:27-11:30')
+        self.assertEqual(tag._convert_to_string(datetime.datetime(1899, 12, 31, 23, 59, 59, tzinfo=FixedOffset('+', 5, 30)), 'Date'),
+                         '1899-12-31T23:59:59+05:30')
+        self.assertEqual(tag._convert_to_string(datetime.datetime(1999, 10, 13, 5, 3, 27, 124300), 'Date'),
+                         '1999-10-13T05:03:27.1243Z')
+        self.assertEqual(tag._convert_to_string(datetime.datetime(1899, 12, 31, 23, 59, 59, 124300), 'Date'),
+                         '1899-12-31T23:59:59.1243Z')
         self.assertEqual(tag._convert_to_string(datetime.datetime(1999, 10, 13, 5, 3, 27, 124300, tzinfo=FixedOffset()), 'Date'),
                          '1999-10-13T05:03:27.1243Z')
+        self.assertEqual(tag._convert_to_string(datetime.datetime(1899, 12, 31, 23, 59, 59, 124300, tzinfo=FixedOffset()), 'Date'),
+                         '1899-12-31T23:59:59.1243Z')
         self.assertEqual(tag._convert_to_string(datetime.datetime(1999, 10, 13, 5, 3, 27, 124300, tzinfo=FixedOffset('+', 5, 30)), 'Date'),
                          '1999-10-13T05:03:27.1243+05:30')
         self.assertEqual(tag._convert_to_string(datetime.datetime(1999, 10, 13, 5, 3, 27, 124300, tzinfo=FixedOffset('-', 11, 30)), 'Date'),
                          '1999-10-13T05:03:27.1243-11:30')
+        self.assertEqual(tag._convert_to_string(datetime.datetime(1899, 12, 31, 23, 59, 59, 124300, tzinfo=FixedOffset('+', 5, 30)), 'Date'),
+                         '1899-12-31T23:59:59.1243+05:30')
         # Invalid values
         self.failUnlessRaises(XmpValueError, tag._convert_to_string, 'invalid', 'Date')
         self.failUnlessRaises(XmpValueError, tag._convert_to_string, None, 'Date')
 
+    def test_convert_to_string_date_with_real_timezones(self):
+        if pytz is None:
+            # Poor man’s test skipping. Starting with Python 2.7, decorators are
+            # available to implement this in a cleaner fashion
+            # (http://docs.python.org/library/unittest.html#unittest-skipping).
+            print 'Install python-tz to run this test. Skipping.'
+            return
+        tag = XmpTag('Xmp.xmp.CreateDate')
+        self.assertEqual(tag.type, 'Date')
+        t = pytz.timezone('UTC').localize(datetime.datetime(2011, 2, 2, 10, 52, 4))
+        self.assertEqual(tag._convert_to_string(t, 'Date'), '2011-02-02T10:52:04Z')
+        t = pytz.timezone('CET').localize(datetime.datetime(2011, 2, 2, 10, 52, 4))
+        self.assertEqual(tag._convert_to_string(t, 'Date'), '2011-02-02T10:52:04+01:00')
+
     def test_convert_to_python_integer(self):
         # Valid values
         tag = XmpTag('Xmp.xmpMM.SaveID')
@@ -318,10 +364,28 @@ class TestXmpTag(unittest.TestCase):
     def test_set_value_empty(self):
         tag = XmpTag('Xmp.dc.creator')
         self.failUnlessEqual(tag.type, 'seq ProperName')
-        self.failUnlessRaises(ValueError, tag._set_value, [])
+        self.failUnlessRaises(ValueError, setattr, tag, 'value', [])
         tag = XmpTag('Xmp.dc.title')
         self.failUnlessEqual(tag.type, 'Lang Alt')
-        self.failUnlessRaises(ValueError, tag._set_value, {})
+        self.failUnlessRaises(ValueError, setattr, tag, 'value', {})
+
+    def test_set_value_incorrect_type(self):
+        # Expecting a list of values
+        tag = XmpTag('Xmp.dc.publisher')
+        self.failUnlessEqual(tag.type, 'bag ProperName')
+        self.failUnlessRaises(TypeError, setattr, tag, 'value', None)
+        self.failUnlessRaises(TypeError, setattr, tag, 'value', 'bleh')
+        # Expecting a dictionary mapping language codes to values
+        tag = XmpTag('Xmp.dc.description')
+        self.failUnlessEqual(tag.type, 'Lang Alt')
+        self.failUnlessRaises(TypeError, setattr, tag, 'value', None)
+        self.failUnlessRaises(TypeError, setattr, tag, 'value', ['bleh'])
+
+    def test_set_value_basestring_for_langalt(self):
+        tag = XmpTag('Xmp.dc.description')
+        self.failUnlessEqual(tag.type, 'Lang Alt')
+        tag.value = 'bleh'
+        self.failUnlessEqual(tag.value, {'x-default': 'bleh'})
 
 
 class TestXmpNamespaces(unittest.TestCase):
